package com.sobot.callsdk.v1.sipphonelibrary;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.os.Build;

import com.sobot.callsdk.R;

import org.linphone.core.LinphoneCall;
import org.linphone.core.LinphoneCallParams;
import org.linphone.core.LinphoneCore;
import org.linphone.core.LinphoneCoreException;
import org.linphone.core.LinphoneCoreFactory;
import org.linphone.core.LinphoneCoreListenerBase;
import org.linphone.core.LpConfig;
import org.linphone.core.PayloadType;
import org.linphone.core.ToneID;
import org.linphone.mediastream.Log;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Timer;
import java.util.TimerTask;

public class SobotPhoneManager extends LinphoneCoreListenerBase {
    private static SobotPhoneManager instance;
    private Context mContext;
    private LinphoneCore mLc;
    private Timer mTimer;
    private LinphoneCall mCall;
    private static boolean sExited;
    private Resources mR;

    private String mLPConfigXsd = null;
    private String mLinphoneFactoryConfigFile = null;
    public String mLinphoneConfigFile = null;
    private String mLinphoneRootCaFile = null;
    private String mRingSoundFile = null;
    private String mChatDatabaseFile = null;


    private BroadcastReceiver mKeepAliveReceiver = new SobotKeepAliveReceiver();

    public SobotPhoneManager(Context serviceContext) {
        mContext = serviceContext;
        sExited = false;
        String basePath = mContext.getFilesDir().getAbsolutePath();
        mLPConfigXsd = basePath + "/lpconfig.xsd";
        mLinphoneFactoryConfigFile = basePath + "/linphonerc";
        mLinphoneConfigFile = basePath + "/.linphonerc";
        mLinphoneRootCaFile = basePath + "/rootca.pem";
        mRingSoundFile = basePath + "/bell_ring.wav";
        mChatDatabaseFile = basePath + "/linphone-history.db";

        mR = serviceContext.getResources();
    }

    public synchronized static final SobotPhoneManager createAndStart(Context context) {
        if (instance != null) {
            throw new RuntimeException("Linphone Manager is already initialized");
        }
        //默认开启日志
        LinphoneCoreFactory.instance().setDebugMode(true, "sobot_sip_log");
        instance = new SobotPhoneManager(context);
        instance.startLibLinphone(context);
        return instance;
    }

    public static synchronized LinphoneCore getLcIfManagerNotDestroyOrNull() {
        if (sExited || instance == null) {
//            SobotLogUtil.e("Trying to get linphone core while SobotPhoneManager already destroyed or not created ");
            return null;
        }
        return getLc();
    }

    public static final boolean isInstanceiated() {
        return instance != null;
    }

    public static synchronized final LinphoneCore getLc() {
        return getInstance().mLc;
    }

    public static synchronized final SobotPhoneManager getInstance() {
        if (instance != null) {
            return instance;
        }
        if (sExited) {
            throw new RuntimeException("Linphone Manager was already destroyed. "
                    + "Better use getLcIfManagerNotDestroyed and check returned value");
        }
        throw new RuntimeException("Linphone Manager should be created before accessed");
    }

    private synchronized void startLibLinphone(Context context) {
        try {
            copyAssetsFromPackage();
            mLc = LinphoneCoreFactory.instance().createLinphoneCore(this, mLinphoneConfigFile,
                    mLinphoneFactoryConfigFile, null, context);
            try {
                initLibLinphone();
            } catch (LinphoneCoreException e) {
//                e.printStackTrace();
            }

            TimerTask task = new TimerTask() {
                @Override
                public void run() {
                    UIThreadDispatcher.dispatch(new Runnable() {
                        @Override
                        public void run() {
                            if (mLc != null) {
                                mLc.iterate();
                            }
                        }
                    });
                }
            };
            mTimer = new Timer("Linphone Scheduler");
            mTimer.schedule(task, 0, 20);
        } catch (Exception e) {
//            e.printStackTrace();
            SobotLogUtil.e("startLibLinphone: cannot start linphone");
        }
    }


    private synchronized void initLibLinphone() throws LinphoneCoreException {
        mLc.setContext(mContext);
        setUserAgent();
        mLc.setRemoteRingbackTone(mRingSoundFile);
        mLc.setTone(ToneID.CallWaiting, mRingSoundFile);
        mLc.setRing(mRingSoundFile);
        mLc.setRootCA(mLinphoneRootCaFile);
        mLc.setChatDatabasePath(mChatDatabaseFile);
//        mLc.setCallErrorTone(Reason.NotFound, mErrorToneFile);//设置呼叫错误播放的铃声

        int availableCores = Runtime.getRuntime().availableProcessors();
        SobotLogUtil.i("MediaStreamer : " + availableCores + " cores detected and configured");
        mLc.setCpuCount(availableCores);

        int migrationResult = getLc().migrateToMultiTransport();
        SobotLogUtil.i("Migration to multi transport result = " + migrationResult);

        mLc.setNetworkReachable(true);

        //回声消除
        boolean isEchoCancellation = true;
        mLc.enableEchoCancellation(isEchoCancellation);

        //自适应码率控制
        boolean isAdaptiveRateControl = true;
        mLc.enableAdaptiveRateControl(isAdaptiveRateControl);

        //audio 码率设置
        getConfig().setInt("audio", "codec_bitrate_limit", 128);

        mLc.setPreferredVideoSizeByName("720p");
        mLc.setUploadBandwidth(1536);
        mLc.setDownloadBandwidth(1536);

        mLc.setVideoPolicy(mLc.getVideoAutoInitiatePolicy(), true);
        mLc.setVideoPolicy(true, mLc.getVideoAutoAcceptPolicy());
        mLc.enableVideo(true, true);

        setCodecMime();

        IntentFilter filter = new IntentFilter(Intent.ACTION_SCREEN_ON);
        filter.addAction(Intent.ACTION_SCREEN_OFF);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            mContext.registerReceiver(mKeepAliveReceiver, filter, Context.RECEIVER_NOT_EXPORTED);
        } else {
            mContext.registerReceiver(mKeepAliveReceiver, filter);
        }
    }


    private void setCodecMime() {
        for (final PayloadType pt : mLc.getVideoCodecs()) {
            SobotLogUtil.i("setCodecMime = " + pt.getMime());
            if (!pt.getMime().equals("VP8")) {
                try {
                    SobotLogUtil.i("disable codec " + pt.getMime());
                    mLc.enablePayloadType(pt, false);
                } catch (LinphoneCoreException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public boolean acceptCallWithParams(LinphoneCall call, LinphoneCallParams params) {
        try {
            mLc.acceptCallWithParams(call, params);
            return true;
        } catch (LinphoneCoreException e) {
            SobotLogUtil.e("Accept call failed");
        }
        return false;
    }

    public static final boolean isInstanciated() {
        return instance != null;
    }

    private void copyAssetsFromPackage() throws IOException {
        copyIfNotExist(R.raw.bell_ring, mRingSoundFile);
        copyIfNotExist(R.raw.linphonerc_default, mLinphoneConfigFile);
        copyIfNotExist(R.raw.linphonerc_factory, new File(mLinphoneFactoryConfigFile).getName());
        copyIfNotExist(R.raw.lpconfig, mLPConfigXsd);
    }


    public void copyIfNotExist(int ressourceId, String target) throws IOException {
        File lFileToCopy = new File(target);
        if (!lFileToCopy.exists()) {
            copyFromPackage(ressourceId, lFileToCopy.getName());
        }
    }

    public void copyFromPackage(int ressourceId, String target) throws IOException {
        FileOutputStream lOutputStream = mContext.openFileOutput(target, 0);
        InputStream lInputStream = mR.openRawResource(ressourceId);
        int readByte;
        byte[] buff = new byte[8048];
        while ((readByte = lInputStream.read(buff)) != -1) {
            lOutputStream.write(buff, 0, readByte);
        }
        lOutputStream.flush();
        lOutputStream.close();
        lInputStream.close();
    }

    private void setUserAgent() {
        try {
            String versionName = mContext.getPackageManager().getPackageInfo(mContext.getPackageName(),
                    0).versionName;
            if (versionName == null) {
                versionName = String.valueOf(mContext.getPackageManager().getPackageInfo(mContext.getPackageName(), 0).versionCode);
            }
            mLc.setUserAgent("sobot_sip_log", versionName);
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
    }

    public static synchronized void destroy() {
        SobotLogUtil.i("-----------SobotPhoneManagger  destroy");
        if (instance == null) {
            return;
        }
        sExited = true;
        instance.doDestroy();
    }

    private void doDestroy() {
        try {
            if (mTimer != null) {
                mLc.destroy();
            }

        } catch (RuntimeException e) {
            e.printStackTrace();
        } finally {
            if (mKeepAliveReceiver != null && mContext != null) {
                mContext.unregisterReceiver(mKeepAliveReceiver);
            }
            mLc = null;
            instance = null;
        }
    }

    public LpConfig getConfig() {
        LinphoneCore lc = getLc();
        if (lc != null) {
            return lc.getConfig();
        }

        if (!SobotPhoneManager.isInstanciated()) {
            File linphonerc = new File(mContext.getFilesDir().getAbsolutePath() + "/.linphonerc");
            if (linphonerc.exists()) {
                return LinphoneCoreFactory.instance().createLpConfig(linphonerc.getAbsolutePath());
            } else if (mContext != null) {
                InputStream inputStream = mContext.getResources().openRawResource(R.raw.linphonerc_default);
                InputStreamReader inputreader = new InputStreamReader(inputStream);
                BufferedReader buffreader = new BufferedReader(inputreader);
                StringBuilder text = new StringBuilder();
                String line;
                try {
                    while ((line = buffreader.readLine()) != null) {
                        text.append(line);
                        text.append('\n');
                    }
                } catch (IOException ioe) {
                    Log.e(ioe);
                }
                return LinphoneCoreFactory.instance().createLpConfigFromString(text.toString());
            }
        } else {
            return LinphoneCoreFactory.instance().createLpConfig(SobotPhoneManager.getInstance().mLinphoneConfigFile);
        }
        return null;
    }
}
