package com.sobot.callsdk.v1.sipphonelibrary;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.text.TextUtils;
import android.widget.Toast;

import com.sobot.callsdk.sip.callback.SobotPhoneCallback;
import com.sobot.callsdk.sip.callback.SobotRegistrationCallback;
import com.sobot.callsdk.v1.sipphonelibrary.permission.ZCPermission;

import org.linphone.core.LinphoneAddress;
import org.linphone.core.LinphoneAuthInfo;
import org.linphone.core.LinphoneCall;
import org.linphone.core.LinphoneCallParams;
import org.linphone.core.LinphoneCore;
import org.linphone.core.LinphoneCoreException;
import org.linphone.core.LinphoneCoreFactory;
import org.linphone.core.LinphoneProxyConfig;

import java.util.List;

public class SobotPhoneUtils {

    private static ServiceWaitThread mServiceWaitThread;
    private static LinphoneCall mCall;
    private static String mUsername, mPassword, mServerIP;
    //传输协议 1 : UDP   2 : TCP    3 : TLS
    private static int mTransportType;

    /**
     * 开启服务
     *
     * @param context 上下文
     */
    public static void startService(Context context) {
        if (!SobotPhoneService.isReady()) {
            Intent intent = new Intent(Intent.ACTION_MAIN);
            intent.setClass(context, SobotPhoneService.class);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                context.startForegroundService(intent);
            } else {
                context.startService(intent);
            }
        }
    }

    /**
     * 关闭服务
     *
     * @param context 上下文
     */
    public static void stopService(Context context) {
        if (SobotPhoneService.isReady()) {
            Intent intent = new Intent(Intent.ACTION_MAIN);
            intent.setClass(context, SobotPhoneService.class);
            context.stopService(intent);
        }
    }


    /**
     * 添加注册状态、通话状态回调
     *
     * @param phoneCallback        通话回调,可为null
     * @param registrationCallback 注册状态回调,可为null
     */
    public static void addPhoneCallStatesback(SobotRegistrationCallback registrationCallback, SobotPhoneCallback phoneCallback) {
        if (SobotPhoneService.isReady()) {
            SobotPhoneService.addRegistrationCallback(registrationCallback);
            SobotPhoneService.addPhoneCallback(phoneCallback);
        } else {
            mServiceWaitThread = new ServiceWaitThread(registrationCallback, phoneCallback);
            mServiceWaitThread.start();
        }
    }


    /**
     * 设置 sip 账户信息
     *
     * @param userName      账号名
     * @param password      密码
     * @param serverIP      IP地址：端口号
     * @param transportType 传输协议 默认UDP方式 1 : UDP   2 : TCP   3 : TLS
     */
    public static void setAccount(String userName, String password, String serverIP, int transportType) {
        mUsername = userName;
        mPassword = password;
        mServerIP = serverIP;
        mTransportType = transportType;
    }


    /**
     * 登录到 SIP 服务器
     */
    public static void login() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                registerUser();
            }
        }).start();
    }


    /**
     * 注册到服务器
     */
    private static void registerUser() {
        logoutUser();
        if (TextUtils.isEmpty(mUsername)) {
            SobotLogUtil.i("账号名 userName  不能为空  ");
            return;
        }
        if (TextUtils.isEmpty(mPassword)) {
            SobotLogUtil.i("密码 password 不能为空  ");
            return;
        }
        if (TextUtils.isEmpty(mServerIP)) {
            SobotLogUtil.i("IP地址端口号 serverIP 不能为空 ");
            return;
        }
        if (mTransportType > 3 || mTransportType < 1) {
            SobotLogUtil.i("传输协议 transportType  只能为 1，2，3");
            return;
        }
        LinphoneCore mLinphoneCore = SobotPhoneManager.getLcIfManagerNotDestroyOrNull();
        if (mLinphoneCore == null) {
            return;
        }
        SobotLogUtil.i("registerUser userName = " + mUsername);
        SobotLogUtil.i("registerUser password = " + mPassword);
        SobotLogUtil.i("registerUser serverIP = " + mServerIP);
        String identify = "sip:" + mUsername + "@" + mServerIP;
        String proxy = "sip:" + mServerIP;
        try {
            LinphoneAddress proxyAddr = LinphoneCoreFactory.instance().createLinphoneAddress(proxy);
            if (mTransportType == 1) {
                proxyAddr.setTransport(LinphoneAddress.TransportType.LinphoneTransportUdp);
            } else if (mTransportType == 2) {
                proxyAddr.setTransport(LinphoneAddress.TransportType.LinphoneTransportTcp);
            } else if (mTransportType == 3) {
                proxyAddr.setTransport(LinphoneAddress.TransportType.LinphoneTransportTls);
            } else {
                proxyAddr.setTransport(LinphoneAddress.TransportType.LinphoneTransportUdp);
            }
            LinphoneAddress identifyAddr = LinphoneCoreFactory.instance().createLinphoneAddress(identify);
            LinphoneAuthInfo authInfo = LinphoneCoreFactory.instance().createAuthInfo(mUsername, null, mPassword,
                    null, null, mServerIP);
            LinphoneProxyConfig prxCfg = mLinphoneCore.createProxyConfig(identifyAddr.asString(),
                    proxyAddr.asStringUriOnly(), proxyAddr.asStringUriOnly(), true);
            prxCfg.enableAvpf(false);
            prxCfg.setAvpfRRInterval(0);
            prxCfg.enableQualityReporting(false);
            prxCfg.setQualityReportingCollector(null);
            prxCfg.setQualityReportingInterval(0);
            prxCfg.enableRegister(true);
            prxCfg.setExpires(31);
            mLinphoneCore.addProxyConfig(prxCfg);
            mLinphoneCore.addAuthInfo(authInfo);
            mLinphoneCore.setDefaultProxyConfig(prxCfg);
        } catch (LinphoneCoreException e) {
            e.printStackTrace();
        }
    }

    /**
     * 注销账户
     */
    public static void logoutUser() {
        LinphoneCore mLinphoneCore = SobotPhoneManager.getLcIfManagerNotDestroyOrNull();
        if (mLinphoneCore == null) {
            return;
        }
        mLinphoneCore.clearProxyConfigs();
        mLinphoneCore.clearAuthInfos();
        if (mServiceWaitThread != null) {
            mServiceWaitThread.stopThread(true);
        }
    }


    /**
     * 拨打电话
     *
     * @param phone 手机号
     */
    public static void startSingleCallingTo(String phone) {
        LinphoneCore mLinphoneCore = SobotPhoneManager.getLcIfManagerNotDestroyOrNull();
        if (mLinphoneCore == null) {
            return;
        }
        LinphoneAddress address = null;
        LinphoneCall call = null;
        try {
            address = mLinphoneCore.interpretUrl(phone);
        } catch (LinphoneCoreException e) {
            e.printStackTrace();
            SobotLogUtil.i(" LinphoneCoreException0:" + e.toString());
        }
        LinphoneCallParams params = mLinphoneCore.createCallParams(null);
        params.setVideoEnabled(false);
        try {
            call = mLinphoneCore.inviteAddressWithParams(address, params);
        } catch (LinphoneCoreException e) {
            e.printStackTrace();
            SobotLogUtil.i(" LinphoneCoreException1:" + e.toString());
        }
    }

    /**
     * 接通电话
     */
    public static void accept(Context context) {
        if (!ZCPermission.checkPermissions(context, new String[]{Manifest.permission.RECORD_AUDIO})) {
            SobotLogUtil.e("没有麦克风危险权限: Manifest.permission.RECORD_AUDIO  ");
            return;
        }
        if (SobotPhoneManager.getLcIfManagerNotDestroyOrNull() != null) {
            List<LinphoneCall> calls = phoneUtils.getLinphoneCalls(SobotPhoneManager.getLc());
            for (LinphoneCall call : calls) {
                if (LinphoneCall.State.IncomingReceived == call.getState()) {
                    mCall = call;
                    break;
                }
            }
        }
        if (mCall == null) {
            SobotLogUtil.i("Couldn't find incoming call");
            return;
        }
        LinphoneCallParams params = SobotPhoneManager.getLc().createCallParams(mCall);
        boolean isLowBandwidthConnection = !phoneUtils.isHighBandwidthConnection(context.getApplicationContext());
        if (params != null) {
            params.enableLowBandwidth(isLowBandwidthConnection);
        } else {
            SobotLogUtil.i("Could not create call params for call");
        }
        if (params == null || !SobotPhoneManager.getInstance().acceptCallWithParams(mCall, params)) {
            Toast.makeText(context, "接电话时发生错误", Toast.LENGTH_LONG).show();
        } else {
            SobotPhoneManager.getLc().enableSpeaker(false);
        }
    }


    /**
     * 挂断电话
     */
    public static void hangUp() {
        LinphoneCore mLinphoneCore = SobotPhoneManager.getLcIfManagerNotDestroyOrNull();
        if (mLinphoneCore == null) {
            return;
        }
        LinphoneCall currentCall = mLinphoneCore.getCurrentCall();
        if (currentCall != null) {
            mLinphoneCore.terminateCall(currentCall);
        } else if (mLinphoneCore.isInConference()) {
            mLinphoneCore.terminateConference();
        } else {
            mLinphoneCore.terminateAllCalls();
        }
    }

    /**
     * 切换静音
     *
     * @param isMicMuted 是否静音
     */
    public static void toggleMicro(boolean isMicMuted) {
        LinphoneCore mLinphoneCore = SobotPhoneManager.getLcIfManagerNotDestroyOrNull();
        if (mLinphoneCore == null) {
            return;
        }
        mLinphoneCore.muteMic(isMicMuted);
    }

    /**
     * 获取免提状态
     *
     * @return true 打开 ; false 关闭
     */
    public static boolean isSpeakerStates() {
        LinphoneCore mLinphoneCore = SobotPhoneManager.getLcIfManagerNotDestroyOrNull();
        if (mLinphoneCore == null) {
            return false;
        }
        return mLinphoneCore.isSpeakerEnabled();
    }


    /**
     * 切换免提
     *
     * @param isSpeakerEnabled 是否免提 true 打开 ; false 关闭
     */
    public static void toggleSpeaker(boolean isSpeakerEnabled) {
        LinphoneCore mLinphoneCore = SobotPhoneManager.getLcIfManagerNotDestroyOrNull();
        if (mLinphoneCore == null) {
            return;
        }
        mLinphoneCore.enableSpeaker(isSpeakerEnabled);
    }

    /**
     * 日志开关
     *
     * @param isOpenLog true 打开 false 关闭 默认打开
     */
    public static void isOpenLog(boolean isOpenLog) {
        if (isOpenLog) {
            SobotLogUtil.showI = true;
            //LinphoneCoreFactory.instance().setDebugMode(true, "sobot_sip_log");
        } else {
            SobotLogUtil.showI = false;
            //LinphoneCoreFactory.instance().setDebugMode(false, "sobot_sip_log");
        }
    }

    private static class ServiceWaitThread extends Thread {
        private SobotPhoneCallback mPhoneCallback;
        private SobotRegistrationCallback mRegistrationCallback;
        private volatile boolean isStop;//使用volatile关键字修饰，可以在多线程之间共享，成员变量来控制线程的停止

        ServiceWaitThread(SobotRegistrationCallback registrationCallback, SobotPhoneCallback phoneCallback) {
            mRegistrationCallback = registrationCallback;
            mPhoneCallback = phoneCallback;
        }

        @Override
        public void run() {
            super.run();
            while (!SobotPhoneService.isReady() && !isStop) {
                try {
                    sleep(80);
                } catch (InterruptedException e) {
                    throw new RuntimeException("waiting thread sleep() has been interrupted");
                }
            }
            SobotPhoneService.addPhoneCallback(mPhoneCallback);
            SobotPhoneService.addRegistrationCallback(mRegistrationCallback);
            mServiceWaitThread = null;
        }

        /**
         * 设置线程标识
         *
         * @param isStop true：停止 false：不停止
         */
        public void stopThread(boolean isStop) {
            this.isStop = isStop;
        }

    }

    public static boolean isServiceRun() {
        if (SobotPhoneService.isReady()) {
            return true;
        } else {
            return false;
        }
    }
}